package com.user.service.token;

import com.alibaba.fastjson.annotation.JSONField;
import com.base.orbit.domain.enity.SysUser;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 定义一个类，实现该接口，把用户信息封装在其中
 * @author xj
 * @date 2022/07/04
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@ToString
public class LoginUser implements UserDetails {

    private SysUser user;

    /**
     * 权限集合
     */
    private List<String> permissions;

    /**
     * 用户唯一标识
     */
    private String token;

    /**
     * 用户生成token值
     */
    private String tokenVal;

    /**
     * 过期时间
     */
    private Long expireTime;

    /**
     * 登录时间
     */
    private Long loginTime;

    //  存储SpringSecurity所需要的权限信息的集合,serialize = false不会将此信息进行序列化
    @JSONField(serialize = false)
    private List<GrantedAuthority> authorities;

    public LoginUser(SysUser user, List<String> permissions){
        this.user=user;
        this.permissions=permissions;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        if(authorities!=null){
            return authorities;
        }
        //把permissions中字符串类型的权限信息转换成GrantedAuthority对象存入authorities中
        authorities = permissions.stream()
                .map(SimpleGrantedAuthority::new)
                .collect(Collectors.toList());
        return authorities;
    }

    @Override
    public String getPassword() {
        return user.getPassWord();
    }

    @Override
    public String getUsername() {
        return user.getUserName();
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }
}